var targetId = "#target",
    oldPicasaWebViewer,
    oldJquery,
testAlbum = {
  title : {
    $t : "myTitle"
  },
  media$group : {
    media$thumbnail : [{url : "myUrl"}]
  },
  link: [{href : "myHref"}]
};

var testPhoto = {
  media$group : {
    media$title : {
      $t : 'varMyTitle'
    },
    media$thumbnail : [{url: "http://www.google.com/image0.png"}, 
      {url : "http://www.google.com/image1.png"}],
  },        
  gphoto$width : {
    $t : 600
  },
  gphoto$height: {
    $t : 400
  },
  content : {
    src : "http://www.google.com/imageBig.png"
  }
};

module("Picasa Web Viewer",{
    setup: function(){
      // reset the html container
      $(targetId).empty();
      // reset the options
      $.picasaWebViewer.overrideOptions({}) ;
    },
    teardown: function(){
    }
});

test("It should set the default options.", function() {
    same($.picasaWebViewer.defaultOptions.urlFormat, 'http://picasaweb.google.com/data/feed/api/user/{0}?alt=json-in-script'); 
    same($.picasaWebViewer.defaultOptions.albumTitleMaxLength,15); 
    same($.picasaWebViewer.defaultOptions.defaultDialogWidth,600); 
    same($.picasaWebViewer.defaultOptions.defaultDialogHeight,400); 

});

test("It should overwrite the default options.", function(){
    $.picasaWebViewer.overrideOptions({
        urlFormat: "http://www.google.com",
        albumTitleMaxLength: 25,
        defaultDialogWidth: 400,
        defaultDialogHeight: 300,
        userName: "HomerSimpon"
    }); 

    same($.picasaWebViewer.options.urlFormat, 'http://www.google.com'); 
    same($.picasaWebViewer.options.albumTitleMaxLength,25); 
    same($.picasaWebViewer.options.defaultDialogWidth,400); 
    same($.picasaWebViewer.options.defaultDialogHeight,300); 
});

test("It should override options when calling the plugin.", function(){
    $(targetId).picasaWebViewer(
      {userName: "BartSimpson"}
    );
    same($.picasaWebViewer.defaultOptions.urlFormat, 'http://picasaweb.google.com/data/feed/api/user/{0}?alt=json-in-script'); 
    same($.picasaWebViewer.options.albumTitleMaxLength,15); 
    same($.picasaWebViewer.options.defaultDialogWidth,600); 
    same($.picasaWebViewer.options.defaultDialogHeight,400); 
    same($.picasaWebViewer.options.userName,"BartSimpson"); 
});

test("It should scaffold the gallery.", function() {
    $.picasaWebViewer.scaffoldGallery($(targetId)); 
    ok($(targetId).find("#gallery").length, "gallery created");
    ok($(targetId).find("#tabs").length, "tabs created");
});

test("It should try to get albums.", function() {
    // Setting up.
    var target = $(targetId)[0] ;		

    // Mock the picasaWebViewer object and the repository.
    var pica = $.picasaWebViewer ;
    var picamocked = sinon.mock(pica);
    picamocked.expects('scaffoldGallery').once();
    picamocked.expects('displayAlbums').never();
    var repo = pica.repository ;
    var repomocked = sinon.mock(repo);
    repomocked.expects('getAlbums').once();

    $.picasaWebViewer.getAndDisplayAlbums(target);

    ok(picamocked.verify(), 'Verify scaffoldGallery was called'); 
    ok(repomocked.verify(), 'Verify displayAlbums was not called'); 

    picamocked.restore();
    repomocked.restore();
});

test("It should get album repository.",function() {

    $.picasaWebViewer.overrideOptions({
        urlFormat: "http://www.google.com/{0}",
        userName: "HomerSimpon"
    }); 

    var mockJquery = sinon.mock(jQuery);
    mockJquery
    .expects("ajax")
    .once()
    .withArgs("http://www.google.com/HomerSimpon")
    .returns("data response");

    $.picasaWebViewer.repository.getAlbums(function() { });    

    ok(mockJquery.verify(), 'Verify ajax was called'); 
    mockJquery.restore();
});

test("It should call display albums.", function() {
    var pica = $.picasaWebViewer,
        picamocked = sinon.mock(pica);
    picamocked.expects('displayAlbum').twice();
    $.picasaWebViewer.displayAlbums([testAlbum, testAlbum]); 
    ok(picamocked.verify(), 'Verify displayAlbum was called twice'); 
    picamocked.restore();
});

test("It should display an album.", function() {
    $.picasaWebViewer.scaffoldGallery($(targetId)[0]);     
    $.picasaWebViewer.displayAlbum(testAlbum); 
    ok($('#gallery')
      .find('li.ui-widget-content')
      .length, 'Found listitem');    
    ok($('#gallery')
      .find('h5')
      .text() === testAlbum.title.$t, 'Title matches');
    ok($('#gallery')
      .find('img')
      .attr('src') === testAlbum.media$group.media$thumbnail[0].url,
      'Thumbnail Url matches');
});

test("It should click on an album.", function() {
    $.picasaWebViewer.scaffoldGallery($(targetId)[0]); 	
    $.picasaWebViewer.displayAlbum(testAlbum);
    var pica = $.picasaWebViewer;
    var picamocked = sinon.mock(pica);
    picamocked
    .expects("getAndDisplayPhotos")
    .once();
    $("#gallery li a").trigger('click');
    ok($('#tabs').tabs('length') > 0, 'A new tab should be added.');	
    ok($('#tabs').find('.ui-tabs-selected a span')
      .text() === testAlbum.title.$t,
      'The title of the span should be correct.');			
    ok(picamocked.verify(), 
      'getAndDisplayPhotos should be called.'); 
    picamocked.restore();
});

test("It should remove an album.", function() {
    // Build the minimum.
    $.picasaWebViewer.scaffoldGallery($(targetId)[0]); 	
    $.picasaWebViewer.displayAlbum(testAlbum); 

    // Mock the display of the photos.
    var pica = $.picasaWebViewer;
    var picamocked = sinon.mock(pica);
    picamocked.expects("getAndDisplayPhotos").once();

    // click on the album then close it.
    $("#gallery li a").trigger('click');
    var tabsLength =  $('#tabs').tabs('length');
    $(".ui-icon-close").trigger('click');

    ok($('#tabs').tabs('length') + 1 === tabsLength, 
      'Old Tab Length should be greater than current tab length.');
    ok($('#tabs').tabs('length') === 1, 'It should deleted the tab.');	
    ok(picamocked.verify(), 'Verify getAndDisplayPhotos was called.'); 

    picamocked.restore();
});

test("It should get and display photos.", function() {
    // Setting up the gallery and the album.
    $.picasaWebViewer.scaffoldGallery($(targetId)[0]); 	
    $.picasaWebViewer.displayAlbum(testAlbum);
    $("#tabs").tabs("add", "#testTab", "myTitle");  

    // Mock the ajax request and photos presentation.
    var pica = $.picasaWebViewer ;
    var picamocked = sinon.mock(pica);
    picamocked
    .expects("displayPhotos")
    .once()
    .withArgs([testPhoto]);
    var repo = pica.repository,
        repomocked = sinon.mock(repo);
    repomocked
    .expects("getPhotos")
    .once()
    .withArgs('http://www.google.com');

    $.picasaWebViewer.getAndDisplayPhotos('http://www.google.com', 
      $("#testTab"));

    ok(repomocked.verify(), 
      'Verify getPhotos was called & returned something.'); 

    repomocked.restore();
    picamocked.restore();
});

test("It should display a photo.", function() {	
    // Setup a gallery and an album.
    $.picasaWebViewer.scaffoldGallery($(targetId)[0]); 	
    $.picasaWebViewer.displayAlbum(testAlbum);
    $("#tabs").tabs("add", "#testTab", "myTitle");  

    // Display a photo.
    $.picasaWebViewer.displayPhoto(testPhoto, $("#testTab")); 

    ok($('#testTab').find('li.ui-widget-content').length, 'Found listitem');	
    ok($('#testTab').find('h5').text() === testPhoto.media$group.media$title.$t, 
      'Title matches');
    ok($('#testTab').find('img')
      .attr('src') === testPhoto.media$group.media$thumbnail[1].url,
      'Thumbnail Url matches');		
});

test("It should click on a photo, and change its size.", function() {
    // Mock the scaling.
    var pica = $.picasaWebViewer ;
    var picamocked = sinon.mock(pica);
    picamocked
    .expects("scaleSize")
    .once()
    .returns({width: 600, height: 400});										

    // setup a gallery, an album, a tab and a picture.
    pica.scaffoldGallery($(targetId)[0]); 	
    pica.displayAlbum(testAlbum);
    $("#tabs").tabs("add", "#testTab", "myTitle").find("#testTab")
    .append("<ul id='testPhoto' class='gallery ui-helper-reset ui-helper-clearfix' />");  
    pica.displayPhoto(testPhoto, $("#testTab ul"));

    $("#testPhoto li").trigger('click');

    ok(picamocked.verify(), 'Verify scaleSize was called'); 

    // Prevent the dialog to stay open.
    $(".ui-dialog-titlebar-close").trigger("click");
    picamocked.restore();
});	

test("It should scale a photo.", function() {
    //Act
    var scaledSize = $.picasaWebViewer.scaleSize({
        maximumWidth : 400, 
        maximumHeight : 600, 
        currentWidth : 400, 
        currentHeight : 600
    });	
    //Assert
    same(scaledSize.width, 400);
    same(scaledSize.height, 600);

    //Act
    scaledSize = $.picasaWebViewer.scaleSize({
        maximumWidth : 400, 
        maximumHeight : 600, 
        currentWidth : 500, 
        currentHeight : 600
    });	
    //Assert	
    same(scaledSize.width, 400);
    same(scaledSize.height, 480);

    //Act
    scaledSize = $.picasaWebViewer.scaleSize({
        maximumWidth : 400, 
        maximumHeight : 600, 
        currentWidth : 600, 
        currentHeight : 400
    });
    //Assert		
    same(scaledSize.width, 400);
    same(scaledSize.height, 266);

    //Act
    scaledSize = $.picasaWebViewer.scaleSize({
        maximumWidth : 400, 
        maximumHeight : 600, 
        currentWidth : 300, 
        currentHeight : 800
    });	
    //Assert	
    same(scaledSize.width, 225);
    same(scaledSize.height, 600);	
});

test("It should truncate some strings.", function() {
    var truncated = $.picasaWebViewer.truncate('short text', 15);
    same(truncated, 'short text');

    truncated = $.picasaWebViewer.truncate('1234567890123', 15);
    same(truncated, '1234567890123');

    truncated = $.picasaWebViewer.truncate('123456789012345', 15);
    same(truncated, '123456789012345');

    truncated = $.picasaWebViewer.truncate('123456789012345678', 15);
    same(truncated, '123456789012...');

    truncated = $.picasaWebViewer.truncate('longer than fifteen', 15);
    same(truncated, 'longer than ...');

    truncated = $.picasaWebViewer.truncate(
      'Really long text that we definately wanted truncated', 15);
    same(truncated, 'Really long ...');	
});

test("It should truncate a title.", function() {
    var truncated = $.picasaWebViewer.truncateTitle('short');
    same(truncated, 'short');

    truncated = $.picasaWebViewer.truncateTitle('123456789012345');
    same(truncated, '123456789012345');

    truncated = $.picasaWebViewer.truncateTitle('123456789012345678');
    same(truncated, '123456789012...');

    truncated = $.picasaWebViewer.truncateTitle('123456789012345678');
    same(truncated, '123456789012...');		

    // Change the option for the split.
    $.picasaWebViewer.overrideOptions({
        albumTitleMaxLength : 10
    });

    var truncated = $.picasaWebViewer.truncateTitle('short');
    same(truncated, 'short');

    truncated = $.picasaWebViewer.truncateTitle('1234567890');
    same(truncated, '1234567890');

    truncated = $.picasaWebViewer.truncateTitle('1234567890123');
    same(truncated, '1234567...');

    truncated = $.picasaWebViewer.truncateTitle('123456789012345678');
    same(truncated, '1234567...');		
});

test("It should call the plug-in.", function() {
    // Mock the scaling.
    var pica = $.picasaWebViewer ;
    var picamocked = sinon.mock(pica);
    picamocked
    .expects("getAndDisplayAlbums")
    .once()
    .withArgs($(targetId)[0]);
    picamocked
    .expects("overrideOptions")
    .once();

    $(targetId).picasaWebViewer({
        userName : "HomerSimpon"
    });

    ok(picamocked.verify(), 
      'Verify overrideOptions & getAndDisplayAlbums called'); 

    picamocked.restore();
});

