(function($){

    var gallery,
        tabs,
        repository,
        picasaWebViewer = $.picasaWebViewer = {} ;

    picasaWebViewer.defaultOptions = {
      urlFormat: "http://picasaweb.google.com/data/feed/api/user/{0}?alt=json-in-script",
      albumTitleMaxLength: 15,
      defaultDialogWidth: 600,
      defaultDialogHeight: 400
    };

    picasaWebViewer.options = null ;

    picasaWebViewer.overrideOptions = function(options){
      picasaWebViewer.options = $.extend(
        {},
        picasaWebViewer.defaultOptions,
        options
      );
    };

    picasaWebViewer.scaffoldGallery = function(element) {
      var html = 
      "<div class='demo ui-widget ui-helper-clearfix'>" +                
      "<div id='tabs'>" + 
      "<ul>" + 
      "<li><a href='#tabs-0'>Albums</a></li>" + 
      "</ul>" + 
      "<div id='tabs-0'>" + 
      "<ul id='gallery' " + 
      "class='gallery ui-helper-reset ui-helper-clearfix' />" + 
      "</div>" + 
      "</div>" +                                                     
      "</div>";
      tabs = $(html).appendTo(element).find('#tabs').tabs({
          add: function(e, ui) {
            $(ui.tab).parents('li:first')
            .append("<span style='float: right;'"+
              " class='ui-icon ui-icon-close' title='Close Tab'></span>")
            .find('.ui-icon-close')
            .click(function() {
                tabs.tabs('remove', $('li', tabs).index($(this)
                    .parents('li:first')[0]));
            });
            tabs.tabs('select', '#' + ui.panel.id);
          }             
        }            	
      );
      gallery = $('#gallery');
    };

    picasaWebViewer.getAndDisplayAlbums = function(element) {
        picasaWebViewer.scaffoldGallery(element);
        repository.getAlbums(function(albums){
            if (albums) {
              picasaWebViewer.displayAlbums(albums);
            }
        });
    };
    
    picasaWebViewer.repository = {

      // retrieve the list of albums from picasa.
      getAlbums : function(callback) {
        var updatedUrl = picasaWebViewer.options.urlFormat.replace("{0}",
          picasaWebViewer.options.userName);
        $.ajax(updatedUrl, { 
            success: function(data) {
              callback(data.feed.entry); 
            },
            dataType: 'jsonp'
        });
      },

      // Get the pictures from Picasa server.
      getPhotos : function(url, callback) {
        $.ajax({ 
            url: url,
            success: function(data) {
              callback(data.feed.entry);
            },
            dataType: 'jsonp'
        });
      }	

    };

    picasaWebViewer.displayAlbums = function(albums) {
      $.each(albums, function() {
          picasaWebViewer.displayAlbum(this);
      });
    };

    // Display an album in the gallery tab.
    picasaWebViewer.displayAlbum = function(album) {
      var title = picasaWebViewer.truncateTitle(album.title.$t);
      var html = 
      "<li class='ui-widget-content ui-corner-tr'>" +
      "<h5 class='ui-widget-header'>" + title + "</h5>" +
      "<a><img src='" + album.media$group.media$thumbnail[0].url +
      "' alt='" + album.title.$t + "' /></a>" +          
      "</li>" ;
      $(html)
        .appendTo(gallery)
        .children("a").attr('href', album.link[0].href)
        .click(picasaWebViewer.clickAlbum);
    };

    picasaWebViewer.truncateTitle = function(title) {
      return picasaWebViewer.truncate(title,
        picasaWebViewer.options.albumTitleMaxLength);
    };

    picasaWebViewer.truncate = function(source, maximumLength) {
      if (source && source.length > maximumLength) {
        source = source.substring(0, maximumLength - 3) + "...";
      }
      return source;
    };

    // Retrieve the pictures from Google and display in the tab.
    picasaWebViewer.getAndDisplayPhotos = function(url, tab) {
      var photoGalleryId = "photos-" + tab.attr("id");
      var photoGallery = $("<ul id='" +
        photoGalleryId +
        "' class='gallery ui-helper-reset ui-helper-clearfix' />")
        .appendTo(tab);
      repository.getPhotos(url, function(photos) {
          if (photos) {
            picasaWebViewer.displayPhotos(photos,  photoGallery);
          }            
      });
    };        

    // Click on an album to create a new tab and show the pictures.
    picasaWebViewer.clickAlbum = function(event) {
      event.preventDefault();
      var newTabIndex = tabs.tabs("length");
      tabs.tabs("add" , "#tabs-" + newTabIndex,
        $(this).find("img").attr("alt"), newTabIndex);     
      var newTab = $("#tabs-"+ newTabIndex);
      tabs.tabs("select", newTabIndex);
      picasaWebViewer.getAndDisplayPhotos(this.href, newTab);           
      return false;                        
    };

    // Display all the pictures of the album in the gallery.
    picasaWebViewer.displayPhotos = function(photos, tabGallery) {
      $.each(photos, function() {
          picasaWebViewer.displayPhoto(this, tabGallery);
      });
    };

    // Display a photo in the album gallery.
    picasaWebViewer.displayPhoto = function(photo, tabGallery) {
      var group = photo.media$group;
      var title = picasaWebViewer.truncateTitle(group.media$title.$t);
      var html = 
      "<li class='ui-widget-content ui-corner-tr'>" +
      "<h5 class='ui-widget-header'>" + title + "</h5>" +
      "<img src='" + group.media$thumbnail[1].url +
      "' alt='" + group.media$title.$t + "' />" +
      "</li>"; 
      $(html)
      .data("metadata", {
          url : photo.content.src,
          width : photo.gphoto$width.$t,            		            		
          height : photo.gphoto$height.$t,
          title : group.media$title.$t
      })
      .appendTo(tabGallery)
      .click($.picasaWebViewer.clickPhoto);
    };

    // Event linked with a photo to open a dialog.
    picasaWebViewer.clickPhoto = function(event) {
      event.preventDefault();
      var metadata = $(this).closest('li').data('metadata')
      var scaledSize = picasaWebViewer.scaleSize({
          maximumWidth : $.picasaWebViewer.options.defaultDialogWidth, 
          maximumHeight : $.picasaWebViewer.options.defaultDialogHeight, 
          currentWidth : metadata.width, 
          currentHeight : metadata.height
      });
      $('<img />').attr({
          'src' : metadata.url,
          'width' : scaledSize.width,
          'height' : scaledSize.height
      }).dialog({
          bgiframe: true,
          autoOpen: false,
          width: scaledSize.width, 
          height: scaledSize.height + 20, 
          modal: true,
          title: metadata.title
      }).dialog('open');
      return false;
    };

    // Set the good dimention for the picture.
    picasaWebViewer.scaleSize = function(dimensions) {
      var ratio = dimensions.currentHeight / dimensions.currentWidth;
      if (dimensions.currentWidth >= dimensions.maximumWidth) {
        dimensions.currentWidth = dimensions.maximumWidth;
        dimensions.currentHeight = dimensions.currentWidth * ratio;
      } else if (dimensions.currentHeight >= dimensions.maximumHeight) {
        dimensions.currentHeight = dimensions.maximumHeight;
        dimensions.currentWidth = dimensions.currentHeight / ratio;
      }
      return {
        width : Math.floor(dimensions.currentWidth),
        height : Math.floor(dimensions.currentHeight)		    
      };
    };

    // Declaration of the jQuery plugin.
    $.fn.picasaWebViewer = function(options){
      repository = picasaWebViewer.repository;
      return this.each(function(){            
          picasaWebViewer.overrideOptions(options);            
          picasaWebViewer.getAndDisplayAlbums(this);
      });
    }; 

})(jQuery);
